
#ifndef __CINT__
#include "TROOT.h"
#include "TSystem.h"
#include "TStyle.h"
#include "TCanvas.h"

#include <iostream>
#include <string>
#include <vector>
#include <cstdlib>
#endif

void AddIncludePath(std::string dir);
void AddLinkedLibs();
void LoadModule(std::string dir);

void rootlogon()
{
  char* c = getenv("G4SMLIBDIR");
  if(!c){
    std::cout << "set environment variable \"G4SMLIBDIR\"" << std::endl;
    std::cout << "quit ROOT" << std::endl;      
    gROOT->ProcessLine(".q");
  }
  std::string install_dir(c);

  AddIncludePath(install_dir);
  AddLinkedLibs();
  LoadModule(install_dir);

  std::cout << "\n Manual of NADEKO \n  http://be.nucl.ap.titech.ac.jp/~kondo/moin/moin.cgi/ANAROOT/Manual/reference#Commands_for_Histogram_Management\n" << std::endl;

  //Base Style
  //  gROOT->SetStyle("Plain");
  gROOT->SetStyle("Modern");
  //  gROOT->SetStyle("Classic");

  //Force Style
  //  gStyle->SetNumberContours(20);
  gStyle->SetHistFillColor(7);
  gStyle->SetHistFillStyle(3002);
  gStyle->SetHistLineColor(kBlue);
  gStyle->SetFuncColor(kRed);
  gStyle->SetFuncWidth(3);
  gStyle->SetCanvasColor(0);
  gStyle->SetCanvasDefH(600);
  gStyle->SetCanvasDefW(600);
  gStyle->SetPadBorderMode(1);
  gStyle->SetPadRightMargin(0.15);
  gStyle->SetPadLeftMargin(0.16);
  gStyle->SetPadTopMargin(0.14);
  gStyle->SetPadBottomMargin(0.14);
  gStyle->SetPadGridX(1);
  gStyle->SetPadGridY(1);
  gStyle->SetFrameLineWidth(2);
  gStyle->SetGridStyle(3);
  gStyle->SetGridColor(16);
  gStyle->SetTitleFillColor(0);
  gStyle->SetTitleStyle(0);
  gStyle->SetTitleY(0.93);
  //  gStyle->SetTitleXOffset(1.);
  gStyle->SetTitleXOffset(1.5);
  gStyle->SetTitleYOffset(2.0);
  gStyle->SetTitleAlign(22);
  gStyle->SetStatColor(0);
  gStyle->SetStatStyle(0);
  //  gStyle->SetStatStyle(1001);
  gStyle->SetStatX(0.848);  
  gStyle->SetStatY(0.86);  
  gStyle->SetStatW(0.3); // SetOptFit(0)で1Dで縦の幅がおかしくなるのを解決できる
  //  gStyle->SetStatH(0.3); 
  //  gStyle->SetStatW(0.2*1.8);
  //  gStyle->SetStatFontSize(0.02);  
  gStyle->SetPalette(1);
  //  gStyle->SetOptLogz(1);
  //  gStyle->SetOptDate(1);
  gStyle->SetStripDecimals(kFALSE);

  if(1){
    gStyle->SetOptTitle(1);
    gStyle->SetOptFit(1);
    gStyle->SetOptStat(1111111);
  }else{
    gStyle->SetOptTitle(0);
    gStyle->SetOptFit(0);
    gStyle->SetOptStat(0);
  }

  gStyle->SetLabelFont(132,"XYZ");
  gStyle->SetLabelSize(0.04, "XYZ");
  gStyle->SetLabelOffset(0.015, "XY");  
  gStyle->SetTitleFont(132,"XYZ");
  gStyle->SetTitleFont(132,"");
  gStyle->SetTextFont(132);
  gStyle->SetStatFont(132);
}

void AddIncludePath(std::string install_dir)
{
  std::vector<std::string> include;
  include.push_back("-I"+install_dir+"/include");
  //  include.push_back("-I/home/ryuki/geant4_workdir/smsimulator/smg4lib/include");
  //  include.push_back("-I/home/ryuki/usr/local/smanalysis/include");
  //  include.push_back("-I/home/ryuki/usr/local/coppola/include");

  std::vector<std::string>::iterator it = include.begin();
  while(it != include.end()){
    gSystem->AddIncludePath((*it).c_str());
    std::cout << "add include path : " << *it << std::endl;
    ++it;
  }
}

void AddLinkedLibs()
{
//  std::vector<std::string> include;
//  include.push_back("`xml2-config --libs`");

//  std::vector<std::string>::iterator it = include.begin();
//  while(it != include.end()){
//    gSystem->AddLinkedLibs((*it).c_str());
//    std::cout << "add linked libs : " << *it << std::endl;
//    ++it;
//  }
}

void LoadModule(std::string install_dir)
{
  std::vector<std::string> modules;
  //  modules.push_back("libXMLParser.so");
  //  modules.push_back(install_dir+"/lib/"+"libanaroot.so");
  modules.push_back(install_dir+"/lib/"+"libsmdata.so");
  modules.push_back(install_dir+"/lib/"+"libsmanalysis.so");
  modules.push_back(install_dir+"/lib/"+"libsmnadeko.so");
  //  modules.push_back("~/geant4_workdir/smsimulator/smg4lib/data/lib/libsmdata.so");
//  modules.push_back("~/geant4_workdir/smsimulator/smg4lib/lib/libsmdata.so");
//  modules.push_back("~/geant4_workdir/smsimulator/smg4lib/lib/libsmanalysis.so");
//  modules.push_back("~/geant4_workdir/smsimulator/smg4lib/lib/libsmnadeko.so");
  //  modules.push_back("~/usr/local/smanalysis/lib/libsmanalysis.so");
  //  modules.push_back("~/usr/local/coppola/lib/libcoppola.so");

  std::vector<std::string>::iterator it = modules.begin();
  while(it != modules.end()){
    std::cout << "reading " << *it << std::endl;
    if(gSystem->Load((*it).c_str()) < 0){
      std::cout << "cannnot read in " << *it << std::endl;      
    }
    ++it;
  }
}

