
#include "NeutronDetectorConstructionMessenger.hh"
#include "NeutronDetectorConstruction.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithAnInteger.hh"
#include "G4UIcmdWith3Vector.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"
#include "G4UIcmdWithoutParameter.hh"

NeutronDetectorConstructionMessenger::NeutronDetectorConstructionMessenger(NeutronDetectorConstruction* Det)
 : fNeutronDetectorConstruction(Det)
{
  fNeutronDetectorDirectory = new G4UIdirectory("/neutrondetector/");
  fNeutronDetectorDirectory->SetGuidance("Modification Commands for Neutron Detector");

  fUpdateCmd = new G4UIcmdWithoutParameter("/neutrondetector/update",this);
  fUpdateCmd->SetGuidance("Update geometry.");
  fUpdateCmd->SetGuidance("This command MUST be applied before \"beamOn\" ");
  fUpdateCmd->SetGuidance("if you changed geometrical value(s)");
  fUpdateCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutNumberCmd = new G4UIcmdWithAnInteger("/neutrondetector/NeutNumber",this);
  fNeutNumberCmd->SetGuidance("Set number of neut in one layer");
  fNeutNumberCmd->SetParameterName("neutNumber",false);
  fNeutNumberCmd->SetRange("neutNumber>0");
  fNeutNumberCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutSizeXCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/NeutSizeX",this);
  fNeutSizeXCmd->SetGuidance("Set X-Size of the neut");
  fNeutSizeXCmd->SetParameterName("neutSizeX",false);
  fNeutSizeXCmd->SetUnitCategory("Length");
  fNeutSizeXCmd->SetRange("neutSizeX>0");
  fNeutSizeXCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutSizeYCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/NeutSizeY",this);
  fNeutSizeYCmd->SetGuidance("Set Y-Size of the neut");
  fNeutSizeYCmd->SetParameterName("neutSizeY",false);
  fNeutSizeYCmd->SetUnitCategory("Length");
  fNeutSizeYCmd->SetRange("neutSizeY>0");
  fNeutSizeYCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutSizeZCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/NeutSizeZ",this);
  fNeutSizeZCmd->SetGuidance("Set Z-Size of the neut");
  fNeutSizeZCmd->SetParameterName("neutSizeZ",false);
  fNeutSizeZCmd->SetUnitCategory("Length");
  fNeutSizeZCmd->SetRange("neutSizeZ>0");
  fNeutSizeZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutGapCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/NeutGap",this);
  fNeutGapCmd->SetGuidance("Set Gap between the neuts");
  fNeutGapCmd->SetParameterName("neutGap",false);
  fNeutGapCmd->SetUnitCategory("Length");
  fNeutGapCmd->SetRange("neutGap>=0");
  fNeutGapCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutRotationModeCmd = new G4UIcmdWithAnInteger("/neutrondetector/NeutRotationMode",this);
  fNeutRotationModeCmd->SetGuidance("Set Rotation mode(1:rotate even neuts pi/2, other:no rotation");
  fNeutRotationModeCmd->SetParameterName("neutRotationMode",false);
  fNeutRotationModeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoNumberCmd = new G4UIcmdWithAnInteger("/neutrondetector/VetoNumber",this);
  fVetoNumberCmd->SetGuidance("Set number of veto in one layer");
  fVetoNumberCmd->SetParameterName("vetoNumber",false);
  fVetoNumberCmd->SetRange("vetoNumber>=0");
  fVetoNumberCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoSizeXCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/VetoSizeX",this);
  fVetoSizeXCmd->SetGuidance("Set X-Size of the veto");
  fVetoSizeXCmd->SetParameterName("vetoSizeX",false);
  fVetoSizeXCmd->SetUnitCategory("Length");
  fVetoSizeXCmd->SetRange("vetoSizeX>0");
  fVetoSizeXCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoSizeYCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/VetoSizeY",this);
  fVetoSizeYCmd->SetGuidance("Set Y-Size of the veto");
  fVetoSizeYCmd->SetParameterName("vetoSizeY",false);
  fVetoSizeYCmd->SetUnitCategory("Length");
  fVetoSizeYCmd->SetRange("vetoSizeY>0");
  fVetoSizeYCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoSizeZCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/VetoSizeZ",this);
  fVetoSizeZCmd->SetGuidance("Set Z-Size of the veto");
  fVetoSizeZCmd->SetParameterName("vetoSizeZ",false);
  fVetoSizeZCmd->SetUnitCategory("Length");
  fVetoSizeZCmd->SetRange("vetoSizeZ>0");
  fVetoSizeZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoGapYCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/VetoGapY",this);
  fVetoGapYCmd->SetGuidance("Set Y-Gap between the vetos");
  fVetoGapYCmd->SetParameterName("vetoGapY",false);
  fVetoGapYCmd->SetUnitCategory("Length");
  fVetoGapYCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoGapZCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/VetoGapZ",this);
  fVetoGapZCmd->SetGuidance("Set Z-Gap between the vetos");
  fVetoGapZCmd->SetParameterName("vetoGapZ",false);
  fVetoGapZCmd->SetUnitCategory("Length");
  fVetoGapZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fVetoRotationModeCmd = new G4UIcmdWithAnInteger("/neutrondetector/VetoRotationMode",this);
  fVetoRotationModeCmd->SetGuidance("Set Rotation mode(1:rotate even vetos pi/2, other:no rotation");
  fVetoRotationModeCmd->SetParameterName("vetoRotationMode",false);
  fVetoRotationModeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutLayerNumberCmd = new G4UIcmdWithAnInteger("/neutrondetector/NeutLayerNumber",this);
  fNeutLayerNumberCmd->SetGuidance("Set number of layer in one stcuk");
  fNeutLayerNumberCmd->SetParameterName("neutLayerNumber",false);
  fNeutLayerNumberCmd->SetRange("neutLayerNumber>0");
  fNeutLayerNumberCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutLayerGapCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/NeutLayerGap",this);
  fNeutLayerGapCmd->SetGuidance("Set gap between the layer");
  fNeutLayerGapCmd->SetParameterName("neutLayerGap",false);
  fNeutLayerGapCmd->SetUnitCategory("Length");
  fNeutLayerGapCmd->SetRange("neutLayerGap>=0");
  fNeutLayerGapCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fNeutVetoGapCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/NeutVetoGap",this);
  fNeutVetoGapCmd->SetGuidance("Set Gap between the Neut and Veto");
  fNeutVetoGapCmd->SetParameterName("neutVetoGap",false);
  fNeutVetoGapCmd->SetUnitCategory("Length");
  fNeutVetoGapCmd->SetRange("neutVetoGap>=0");
  fNeutVetoGapCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fStuckNumberCmd = new G4UIcmdWithAnInteger("/neutrondetector/StuckNumber",this);
  fStuckNumberCmd->SetGuidance("Set number of stuck");
  fStuckNumberCmd->SetParameterName("stuckNumber",false);
  fStuckNumberCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fStuckGapCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/StuckGap",this);
  fStuckGapCmd->SetGuidance("Set Gap between the stucks");
  fStuckGapCmd->SetParameterName("stuckGap",false);
  fStuckGapCmd->SetUnitCategory("Length");
  fStuckGapCmd->SetRange("stuckGap>=0");
  fStuckGapCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fStuckIntervalCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/StuckInterval",this);
  fStuckIntervalCmd->SetGuidance("Set Interval of the stucks. MUST be set after other geometory fix.");
  fStuckIntervalCmd->SetParameterName("stuckInterval",false);
  fStuckIntervalCmd->SetUnitCategory("Length");
  fStuckIntervalCmd->SetRange("stuckInterval>0");
  fStuckIntervalCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fPositionXCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/PositionX",this);
  fPositionXCmd->SetGuidance("Set X-Pos of the Detector");
  fPositionXCmd->SetParameterName("positionX",false);
  fPositionXCmd->SetUnitCategory("Length");
  fPositionXCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fPositionYCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/PositionY",this);
  fPositionYCmd->SetGuidance("Set Y-Pos of the Detector");
  fPositionYCmd->SetParameterName("positionY",false);
  fPositionYCmd->SetUnitCategory("Length");
  fPositionYCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fPositionZCmd = new G4UIcmdWithADoubleAndUnit("/neutrondetector/PositionZ",this);
  fPositionZCmd->SetGuidance("Set Z-Pos of the Detector");
  fPositionZCmd->SetParameterName("positionZ",false);
  fPositionZCmd->SetUnitCategory("Length");
  fPositionZCmd->SetRange("positionZ>=0");
  fPositionZCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
}

NeutronDetectorConstructionMessenger::~NeutronDetectorConstructionMessenger()
{
  delete fNeutronDetectorDirectory;

  delete fUpdateCmd;

  delete fNeutNumberCmd;
  delete fNeutSizeXCmd;
  delete fNeutSizeYCmd;
  delete fNeutSizeZCmd;
  delete fNeutGapCmd;
  delete fNeutRotationModeCmd;

  delete fVetoNumberCmd;
  delete fVetoSizeXCmd;
  delete fVetoSizeYCmd;
  delete fVetoSizeZCmd;
  delete fVetoGapYCmd;
  delete fVetoGapZCmd;
  delete fVetoRotationModeCmd;

  delete fNeutLayerNumberCmd;
  delete fNeutLayerGapCmd;

  delete fNeutVetoGapCmd;

  delete fStuckNumberCmd;
  delete fStuckGapCmd;
  delete fStuckIntervalCmd;

  delete fPositionXCmd;
  delete fPositionYCmd;
  delete fPositionZCmd;
}

void NeutronDetectorConstructionMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
  if(command == fUpdateCmd){
    fNeutronDetectorConstruction->UpdateGeometry();

  }else if(command == fNeutNumberCmd){
    fNeutronDetectorConstruction->SetNeutNumber(fNeutNumberCmd->GetNewIntValue(newValue));
  }else if(command == fNeutSizeXCmd){
    fNeutronDetectorConstruction->SetNeutSizeX(fNeutSizeXCmd->GetNewDoubleValue(newValue));
  }else if(command == fNeutSizeYCmd){
    fNeutronDetectorConstruction->SetNeutSizeY(fNeutSizeYCmd->GetNewDoubleValue(newValue));
  }else if(command == fNeutSizeZCmd){
    fNeutronDetectorConstruction->SetNeutSizeZ(fNeutSizeZCmd->GetNewDoubleValue(newValue));
  }else if(command == fNeutGapCmd){
    fNeutronDetectorConstruction->SetNeutGap(fNeutGapCmd->GetNewDoubleValue(newValue));
  }else if(command == fNeutRotationModeCmd){
    fNeutronDetectorConstruction->SetNeutRotationMode(fNeutRotationModeCmd->GetNewIntValue(newValue));

  }else if(command == fVetoNumberCmd){
    fNeutronDetectorConstruction->SetVetoNumber(fVetoNumberCmd->GetNewIntValue(newValue));
  }else if(command == fVetoSizeXCmd){
    fNeutronDetectorConstruction->SetVetoSizeX(fVetoSizeXCmd->GetNewDoubleValue(newValue));
  }else if(command == fVetoSizeYCmd){
    fNeutronDetectorConstruction->SetVetoSizeY(fVetoSizeYCmd->GetNewDoubleValue(newValue));
  }else if(command == fVetoSizeZCmd){
    fNeutronDetectorConstruction->SetVetoSizeZ(fVetoSizeZCmd->GetNewDoubleValue(newValue));
  }else if(command == fVetoGapYCmd){
    fNeutronDetectorConstruction->SetVetoGapY(fVetoGapYCmd->GetNewDoubleValue(newValue));
  }else if(command == fVetoGapZCmd){
    fNeutronDetectorConstruction->SetVetoGapZ(fVetoGapZCmd->GetNewDoubleValue(newValue));
  }else if(command == fVetoRotationModeCmd){
    fNeutronDetectorConstruction->SetVetoRotationMode(fVetoRotationModeCmd->GetNewIntValue(newValue));

  }else if(command == fNeutLayerNumberCmd){
    fNeutronDetectorConstruction->SetNeutLayerNumber(fNeutLayerNumberCmd->GetNewIntValue(newValue));
  }else if(command == fNeutLayerGapCmd){
    fNeutronDetectorConstruction->SetNeutLayerGap(fNeutLayerGapCmd->GetNewDoubleValue(newValue));

  }else if(command == fNeutVetoGapCmd){
    fNeutronDetectorConstruction->SetNeutVetoGap(fNeutVetoGapCmd->GetNewDoubleValue(newValue));

  }else if(command == fStuckNumberCmd){
    fNeutronDetectorConstruction->SetStuckNumber(fStuckNumberCmd->GetNewIntValue(newValue));
  }else if(command == fStuckGapCmd){
    fNeutronDetectorConstruction->SetStuckGap(fStuckGapCmd->GetNewDoubleValue(newValue));
  }else if(command == fStuckIntervalCmd){
    fNeutronDetectorConstruction->SetStuckInterval(fStuckIntervalCmd->GetNewDoubleValue(newValue));

  }else if(command == fPositionXCmd){
    fNeutronDetectorConstruction->SetPositionX(fPositionXCmd->GetNewDoubleValue(newValue));
  }else if(command == fPositionYCmd){
    fNeutronDetectorConstruction->SetPositionY(fPositionYCmd->GetNewDoubleValue(newValue));
  }else if(command == fPositionZCmd){
    fNeutronDetectorConstruction->SetPositionZ(fPositionZCmd->GetNewDoubleValue(newValue));
  }
}


