
#include "TAnalysisTools.hh"

#include <iostream>
#include <string>
#include <cmath>

ClassImp(TAnalysisTools)

//demons like
Double_t TAnalysisTools::MeVeeRelation(Double_t T, std::string name, Int_t pdgcode)
{
  Double_t a1, a2, a3, a4;

  if(name == "e-" || name == "e+" || name == "mu-" || name == "mu+"){//leptons
    a1 = 1;
    a2 = 0;
    a3 = 0;
    a4 = 0;
  }else if(name == "proton" || name == "deuteron" || name == "triton"){//Z=1
    //NE-102(CECIL et al.) 
    a1 = 0.95;
    a2 = 8.0;
    a3 = 0.1;
    a4 = 0.9;
  }else if(name == "alpha" || name == "He3"){//Z=2
    //plastic(CECIL et al.)
    a1 = 0.41;
    a2 = 5.9;
    a3 = 0.065;
    a4 = 1.01;
  }else{//heavy ions
    //used in DEMONS
    a1 = 0.29;
    a2 = 37.492;
    a3 = 0.01;
    a4 = 0.874;
  }

  return a1*T-a2*(1-exp(-a3*pow(T, a4)));
}

//fit by
// D.Fox et al.    Nucl. Inst. and Meth. A374(1996)63 
Double_t TAnalysisTools::MeVeeRelation2(Double_t T, std::string name, Int_t pdgcode)
{
  Double_t a1, a2, a3, a4;

  //    std::cout << pdgcode << std::endl;
  //  std::cout << name << " " << pdgcode << std::endl;
  if(name == "e-" || name == "e+" || name == "mu-" || name == "mu+"){//leptons
    a1 = 1;
    a2 = 0;
    a3 = 0;
    a4 = 0;
  }else if(name == "proton"){
    a1 = 0.902713 ;
    a2 = 7.55009  ;
    a3 = 0.0990013;
    a4 = 0.736281 ;
  }else if(name == "deuteron"){
    a1 = 0.891575 ;
    a2 = 12.2122  ;
    a3 = 0.0702262;
    a4 = 0.782977 ;
  }else if(name == "triton"){
    a1 = 0.881489 ;
    a2 = 15.9064  ;
    a3 = 0.0564987;
    a4 = 0.811916 ;
  }else if(name == "He3"){
    a1 = 0.803919 ;
    a2 = 34.4153  ;
    a3 = 0.0254322;
    a4 = 0.894859 ;
  }else if(name == "alpha"){
    a1 = 0.781501 ;
    a2 = 39.3133  ;
    a3 = 0.0217115;
    a4 = 0.910333 ;
  }else{
    pdgcode -= 1000000000;
    double z = pdgcode / 10000;
    //    std::cout << z << std::endl;
    //    double a = pdgcode % 10000;
    if(z == 3){ // Li
      // Li7
      a1 = 0.613491 ;
      a2 = 57.1372  ;
      a3 = 0.0115948;
      a4 = 0.951875 ;
    }else if(z == 4){ // be
      // Be9
      a1 = 0.435772 ;
      a2 = 45.538   ;
      a3 = 0.0104221;
      a4 = 0.916373 ;
    }else if(z == 5){ // B
      // B11
      a1 = 0.350273 ;
      a2 = 34.4664  ;
      a3 = 0.0112395;
      a4 = 0.912711 ;
    }else if(z == 6){ // C
      // C12
      a1 = 0.298394 ;
      a2 = 25.5679  ;
      a3 = 0.0130345; 
      a4 = 0.908512 ;
    }else{
      // C12
      a1 = 0.298394 ;
      a2 = 25.5679  ;
      a3 = 0.0130345; 
      a4 = 0.908512 ;
    }
  }
  return a1*T-a2*(1-exp(-a3*pow(T, a4)));
}

Double_t TAnalysisTools::MeVtoMeVee(Double_t Tin, Double_t Tout, std::string name, Int_t pdgcode)
{
  Double_t Tein, Teout, dTe;

  Tein = MeVeeRelation2(Tin, name, pdgcode);
  Teout = MeVeeRelation2(Tout, name, pdgcode);
  dTe = Tein - Teout;

  if(dTe < 0) dTe = 0;

  return dTe;
}
