
#include "DetectorConstructionExampleDALI2.hh"
#include "DetectorConstructionExampleDALI2Messenger.hh"

#include "SourceConstruction.hh"
#include "BeamPipeConstruction.hh"
#include "DALI2Construction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"

#include "G4RunManager.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

DetectorConstructionExampleDALI2::DetectorConstructionExampleDALI2()
  : fLogicWorld(0), fLogicSource(0), fLogicBeamPipe(0), fLogicDALI2(0)
{
  fDetectorConstructionExampleDALI2Messenger = new DetectorConstructionExampleDALI2Messenger(this);

  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");

  fSourceConstruction = new SourceConstruction;
  fBeamPipeConstruction = new BeamPipeConstruction;
  fDALI2Construction = new DALI2Construction;
}

DetectorConstructionExampleDALI2::~DetectorConstructionExampleDALI2()
{
  delete fSourceConstruction;
  delete fBeamPipeConstruction;
  delete fDALI2Construction;
}

G4VPhysicalVolume* DetectorConstructionExampleDALI2::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  fLogicSource = fSourceConstruction->ConstructSub();
  fLogicBeamPipe = fBeamPipeConstruction->ConstructSub();
  fLogicDALI2 = fDALI2Construction->ConstructSub();

  // Position
  G4ThreeVector sourcePos(0, 0, 0);
  G4ThreeVector beamPipePos(0, 0, 176*mm);
  G4ThreeVector dali2Pos(0, 0, 0);

  // World LV
  G4ThreeVector dali2Size = fDALI2Construction->GetDALI2Size();
  G4ThreeVector worldSize(1.2*dali2Size.x(),
			  1.2*dali2Size.y(),
			  1.2*dali2Size.z());
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // source PV
  new G4PVPlacement(0,
		    sourcePos,
		    fLogicSource,
		    "Source",
		    fLogicWorld,
		    false,
		    0);

  // beam pipe PV
  new G4PVPlacement(0,
		    beamPipePos,
		    fLogicBeamPipe,
		    "BeamPipe",
		    fLogicWorld,
		    false,
		    0);

  // DALI2 PV
  new G4PVPlacement(0,
		    dali2Pos,
		    fLogicDALI2,
		    "DALI2",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicDALI2->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

void DetectorConstructionExampleDALI2::UpdateGeometry()
{
  G4RunManager::GetRunManager()->DefineWorldVolume(Construct());
}

